/////////////////////////////////////////////////////////////////////////
// -  Drag and Drop
function DragObject (element) {
 element.dragObject = this;
 dragMaster.makeDraggable(element);
 var rememberPosition;
 var mouseOffset;
 this.onDragStart = function(offset) {
  var s = element.style;
  rememberPosition = {top: s.top, left: s.left, position: s.position};
  s.position = 'absolute';
  mouseOffset = offset;
 }
 this.hide = function() {
  element.style.display = 'none' ;
 }
 this.show = function() {
  element.style.display = '';
 }
 this.onDragMove = function(x, y) {
  element.style.top =  y - mouseOffset.y +'px';
  element.style.left = x - mouseOffset.x +'px';
 }
 this.onDragSuccess = function(dropTarget) { 
  //alert (dropTarget+' '+this.toString()); //ex.: e4, wr8
 }
 this.onDragFail = function() {
  var s = element.style;
  s.top = rememberPosition.top;
  s.left = rememberPosition.left;
  s.position = rememberPosition.position;
 }
 this.toString = function() {
  return element.id;
 }
}

function DropTarget (element) {
 element.dropTarget = this;
 this.canAccept = function(dragObject) {  //     
  return true;
 }
 this.accept = function(dragObject) {
  this.onLeave();
//  dragObject.hide();
//  alert(" '"+this+"':   '"+dragObject+"'"); //ex.: this=a1, dragObject=bq8
  var fields = document.getElementById('bTable').getElementsByTagName('img');
  var dragName = dragObject.toString();
  for (var i=0; i<fields.length; i++) {
   if (fields[i].alt == dragName) { fields[i].alt=''; break; } //!!!
  }
  document.getElementById(this).setAttribute('alt',dragName);
  document.getElementById('fen').value=genFen();
 }
 this.onLeave = function() { element.className =  ''; }
 this.onEnter = function() { element.className = 'uponMe'; }
 this.toString = function() { return element.id; }
}

var dragMaster = (function() {
 var dragObject;
 var mouseDownAt;
 var currentDropTarget;
 function mouseDown(e) { //   :  
  e = fixEvent(e);
  if (e.which!=1) return;
  mouseDownAt = { x: e.pageX, y: e.pageY, element: this }
  addDocumentEventHandlers();
  return false;
 }
 function mouseMove(e) {
  e = fixEvent(e)
  // (1)
  if (mouseDownAt) { //    
   if (Math.abs(mouseDownAt.x-e.pageX)<5 && Math.abs(mouseDownAt.y-e.pageY)<5) {
    return false;
   }
   //  
   var elem  = mouseDownAt.element; //    
   dragObject = elem.dragObject; // ,      
   var mouseOffset = getMouseOffset(elem, mouseDownAt.x, mouseDownAt.y);
   mouseDownAt = null; //     ,   
   dragObject.onDragStart(mouseOffset); // 
  }
  // (2)
  dragObject.onDragMove(e.pageX, e.pageY);
  // (3)
  var newTarget = getCurrentTarget(e);
  // (4)
  if (currentDropTarget != newTarget) {
   if (currentDropTarget) { currentDropTarget.onLeave(); }
   if (newTarget) { newTarget.onEnter(); }
   currentDropTarget = newTarget;
  }
  // (5)
  return false;
 }
 function mouseUp() { // 
  if (!dragObject) { // (1)
   mouseDownAt = null;
  } 
  else { // (2)
   if (currentDropTarget) {
    currentDropTarget.accept(dragObject);
    dragObject.onDragSuccess(currentDropTarget);
  } 
  else { dragObject.onDragFail(); }
  dragObject = null;
 }
 // (3)
 removeDocumentEventHandlers();
}

function getMouseOffset(target, x, y) {
 var docPos = getOffset(target);
 return {x:x - docPos.left, y:y - docPos.top};
}
function getCurrentTarget(e) {
 //  ,     -     
 if (navigator.userAgent.match('MSIE') || navigator.userAgent.match('Gecko')) {
  var x=e.clientX, y=e.clientY;
 } 
 else {	var x=e.pageX, y=e.pageY; }
 //      -     hide  show
 dragObject.hide();
 var elem = document.elementFromPoint(x,y);
 dragObject.show();
 //    dropTarget
 while (elem) { //    dragObject 
  if (elem.dropTarget && elem.dropTarget.canAccept(dragObject)) {
   return elem.dropTarget;
  }
  elem = elem.parentNode;
 }
 return null; //dropTarget  
}
function addDocumentEventHandlers() { //    
 document.onmousemove = mouseMove;
 document.onmouseup = mouseUp;                                                 
 document.ondragstart = document.body.onselectstart = function() { return false };
}
function removeDocumentEventHandlers() { //  
 document.onmousemove = document.onmouseup = document.ondragstart = document.body.onselectstart = null;
}
return { makeDraggable: function(element) { element.onmousedown = mouseDown; } }
}())

function fixEvent(e) { //   ""  IE
 e = e || window.event;
 //  pageX/pageY  IE
 if ( e.pageX == null && e.clientX != null ) {
  var html = document.documentElement;
  var body = document.body;
  e.pageX = e.clientX + (html && html.scrollLeft || body && body.scrollLeft || 0) - (html.clientLeft || 0);
  e.pageY = e.clientY + (html && html.scrollTop || body && body.scrollTop || 0) - (html.clientTop || 0);
 }
 //  which  IE
 if (!e.which && e.button) {
  e.which = e.button & 1 ? 1 : ( e.button & 2 ? 3 : ( e.button & 4 ? 2 : 0 ) );
 }
 return e;
}

function getOffset(elem) {
 if (elem.getBoundingClientRect) { return getOffsetRect(elem); } 
 else { return getOffsetSum(elem); }
}

function getOffsetRect(elem) {
 var box = elem.getBoundingClientRect();
 var body = document.body;
 var docElem = document.documentElement;
 var scrollTop = window.pageYOffset || docElem.scrollTop || body.scrollTop;
 var scrollLeft = window.pageXOffset || docElem.scrollLeft || body.scrollLeft;
 var clientTop = docElem.clientTop || body.clientTop || 0;
 var clientLeft = docElem.clientLeft || body.clientLeft || 0;
 var top  = box.top +  scrollTop - clientTop;
 var left = box.left + scrollLeft - clientLeft;
 return { top: Math.round(top), left: Math.round(left) };
}

function getOffsetSum(elem) {
 var top=0, left=0
 while(elem) {
  top = top + parseInt(elem.offsetTop);
  left = left + parseInt(elem.offsetLeft);
  elem = elem.offsetParent;
 }
 return {top: top, left: left};
}

/////////////////////////////////////////////////////////////////////////
// -   
var IMGLink=
 "http://pers.narod.ru/jscript/ChessPositionBuilder/";
var FENLink=
 "http://scripts.kislenko.net/genfen/index.php?fen=";

 function printFigures (color) {
  var pr = new Array ('r','n','b','q','k','b','n','r');
  var pn = new Array (1,1,1,1,1,2,2,2);
  var i,id;
  document.writeln ('<tr><td>');
  for (i=1; i<9; i++) {
   id=color+pr[i-1];
   document.writeln ('<img src="'+IMGLink+'img/'+id+'.png" id="'+(id+pn[i-1])+'">');
  }
  document.writeln ('</td></tr>');
 }

 function printPawns (color) {
  var i,id;
  document.writeln ('<tr><td>');
  for (i=1; i<9; i++) {
   id=color+'p';
   document.writeln ('<img src="'+IMGLink+'img/'+id+'.png" id="'+(id+i)+'">');
  }
  document.writeln ('</td></tr>');
 }

 function printFiguresDiv () {
  document.writeln ('<div id="dragObjects"><table cellpadding="0" cellspacing="0" border="0" align="center">');
  printFigures ('b');
  printPawns ('b');
  printPawns ('w');
  printFigures ('w');
  document.writeln ('</table></div>');
 }

 var Board=new Array ( 
  new Array ('a8','b8','c8','d8','e8','f8','g8','h8'),
  new Array ('a7','b7','c7','d7','e7','f7','g7','h7'),
  new Array ('a6','b6','c6','d6','e6','f6','g6','h6'),
  new Array ('a5','b5','c5','d5','e5','f5','g5','h5'),
  new Array ('a4','b4','c4','d4','e4','f4','g4','h4'),
  new Array ('a3','b3','c3','d3','e3','f3','g3','h3'),
  new Array ('a2','b2','c2','d2','e2','f2','g2','h2'),
  new Array ('a1','b1','c1','d1','e1','f1','g1','h1')
 );

 var fieldColorWhite='#EEEEEE';
 var fieldColorBlack='#888888';
 var framesColor='green';

 function printBoardDiv () {
  document.writeln ('<div id="bTable"><table cellpadding="0" cellspacing="0" border="1" align="center">');
  for (var i=0; i<8; i++) {
   var color=(i%2==0?fieldColorWhite:fieldColorBlack);
   document.writeln ('<tr>');
   for (var j=0; j<8; j++) {
    document.writeln ('<td bgcolor="'+color+'"><img src="'+IMGLink+'img/blank.png" id="'+Board[i][j]+'" alt="'+Board[i][j]+'" title="'+Board[i][j]+'"></td>');
    color = (color==fieldColorWhite?fieldColorBlack:fieldColorWhite);
   }
   document.writeln ('</tr>');
  }
  document.writeln ('</table></div>');
 }

 document.writeln ('<style>.uponMe { background-color: #ef98c3; }</style>');
 document.writeln ('<div id="freeSpaces"><table align="center" valign="middle" cellpadding="0" cellspacing="0" border="1" bordercolor="'+framesColor+'" nowrap><tr><td align="center" valign="middle">');
 printBoardDiv ();
 document.writeln ('</td></tr></table></div>');
 printFiguresDiv ();

 document.writeln ('<p align="center"> ,    :<br>');
 var recentFigures=new Array ('wr','wn','wb','wq','bq','bb','bn','br');
 var countFigures=new Array (3,3,3,2,2,3,3,3);

 for (var i=0; i<recentFigures.length; i++) {
  var id= recentFigures[i]+'0';
  document.writeln ('<a href="#" onclick="javascript:addFigure(\''+recentFigures[i]+'\');"><img src="'+IMGLink+'img/'+recentFigures[i]+'.png" id="'+id+'" border="0"></a>');
 }
 document.writeln ('</p>');

 onload = function() {
  var dragObjects = document.getElementById('dragObjects').getElementsByTagName('img');
  for(var i=0; i<dragObjects.length; i++) {
   new DragObject(dragObjects[i]);
  }
  for (var i=0; i<8; i++) {
   for (var j=0; j<8; j++) {
    new DropTarget(document.getElementById(Board[i][j]));
   }
  }   

  new DropTarget(document.getElementById('freeSpaces'));
  new DropTarget(document.getElementById('bTable'));
 }
 
 function addFigure (f) {
  var num=0;  
  for (var i=0; i<recentFigures.length; i++) {
   if (f==recentFigures[i]) { num=countFigures[i]++; break; }
  }
  if (num==0) return;
  var bid=document.getElementById('dragObjects');
  var image=document.createElement('img');  
  image.src=IMGLink+'img/'+f+'.png';  
  image.id=f+num;
  bid.appendChild(image);  
  new DragObject(image);
 }

// FEN 
function genFen () {
 result="";
 count = 0;
 var fields = document.getElementById('bTable').getElementsByTagName('img');
 for (var i=0; i<fields.length; i++) {
  if (i%8==0 && i!=0) {
   if (count) result += count;
   result += '/';
   count = 0;
  }
  switch (fields[i].alt.substring(0,2)) {
   case 'wk': if (count) result += count; result += 'K'; count = 0; break;
   case 'wq': if (count) result += count; result += 'Q'; count = 0; break;
   case 'wr': if (count) result += count; result += 'R'; count = 0; break;
   case 'wb': if (count) result += count; result += 'B'; count = 0; break;
   case 'wn': if (count) result += count; result += 'N'; count = 0; break;
   case 'wp': if (count) result += count; result += 'P'; count = 0; break;
   case 'bk': if (count) result += count; result += 'k'; count = 0; break;
   case 'bq': if (count) result += count; result += 'q'; count = 0; break;
   case 'br': if (count) result += count; result += 'r'; count = 0; break;
   case 'bb': if (count) result += count; result += 'b'; count = 0; break;
   case 'bn': if (count) result += count; result += 'n'; count = 0; break;
   case 'bp': if (count) result += count; result += 'p'; count = 0; break;
   default: count++;
  }
 }
 if (count) result += count;
 document.getElementById('dlink').setAttribute("href", FENLink+result);
 document.getElementById('dlink').style.display = '';
 document.getElementById('dcode').style.display = '';
 document.getElementById('fen').innerHTML=result;
 return result;
}

//  
function getCodes () {
 pr = window.open("", "pr",
  "toolbar=no,scrollbars=yes,directories=no,status=no,menubar=yes,resizable=yes,width=650,height=400");
 pr.document.open();
 pr.document.writeln ('<HTML><HEAD><TITLE>  FEN</TITLE></HEAD><BODY><p> HTML   :<form><textarea cols="72" rows="3">');
 pr.document.writeln ('&lt;img src="'+FENLink+document.getElementById('fen').innerHTML+'" border="0"&gt;');
 pr.document.writeln('</textarea></form><p> PHPbb    :<form><textarea cols="72" rows="3">');
 pr.document.writeln('[img]'+FENLink+document.getElementById('fen').innerHTML+'[/img]');
 pr.document.write ('</textarea></form></BODY></HTML>');
 pr.document.close();
}